import axios from 'axios'
import {
    INIT_PAYMENTS_URL,
    PAYMENTS_PROCESS_URL
} from '@/constants/resourcePaths'
/*Action */
import {
    PAYMENTS_NOTIFY,
    INIT_PAYMENTS,
    EXPLODE_DATA,
    INIT_PAYMENTS_DATE,
    INIT_PAYMENTS_USER,
    INIT_PAYMENTS_CUSTOMERS,
    INIT_PAYMENTS_JOURNALS,
    INIT_PAYMENTS_CURRENCIES,
    PAYMENTS_PROCESS,
    CHECK_CUSTOMER,
    CHECK_INVOICE,
    CHECK_MOVE,
    CHECK_PAYMENTS,
    COMPLETED_PAYMENTS,
    RESET_PAYMENT_CUSTOMER,
    RESET_DATE,
    RESET_USER,
    RESET_COMPANY,
    RESET_CUSTOMERS,
    RESET_INVOICES,
    RESET_MOVE_LINES,
    RESET_JOURNALS,
    RESET_CURRENCY,
    RESET_PAYMENTS
} from '@/constants/actionTypes'
// Mutations
import {
    SET_PROCESSING_PAYMENTS,
    SET_COMPLETED_PAYMENTS
} from '@/constants/mutationTypes'

const actions = {
    /**
     * [PAYMENTS_NOTIFY]
     * @type {[type]}
     */
    [PAYMENTS_NOTIFY] ({ commit },payload) {
        openerp.web.notification.do_warn('Atencion', payload)
        return false
    },
    /**
     * [INIT_PAYMENTS]
     * @param {*} param0
     */
    [INIT_PAYMENTS] ({ dispatch }) {
        return axios.get(INIT_PAYMENTS_URL).then(response => {
            dispatch(COMPLETED_PAYMENTS, false)
            dispatch('explodeData', response.data)
        }).catch(error => {
            console.log(error);
        })
    },
    /**
     * [EXPLODE_DATA]
     * @param {*} param0
     * @param {*} payload
     */
    [EXPLODE_DATA] ({ dispatch }, payload) {
        for (let value in payload) {
            dispatch(`initPayments${value[0].toUpperCase()}${value.slice(1)}`,payload[value])
        }
    },
    /**
     *
     */
    [CHECK_CUSTOMER] ({ getters, dispatch }) {
        return !!getters.selectedCustomer ||dispatch(PAYMENTS_NOTIFY,'Necesitas seleccionar un cliente para continuar.')
    },
    /**
     *
     */
    [CHECK_INVOICE] ({ getters, dispatch }) {
        return !!getters.selectedInvoice ||dispatch(PAYMENTS_NOTIFY, 'Necesitas seleccionar una factura para continuar.')
    },
    /**
     *
     */
    [CHECK_MOVE] ({ getters, dispatch }) {
        return !!getters.movesPayments.length || dispatch(PAYMENTS_NOTIFY,'Necesitas seleccionar al menos una cuota para continuar.')
    },
    /**
     *
     */
    [CHECK_PAYMENTS] ({ getters, dispatch }) {
        if (!getters.selectedJournal) {
             return dispatch(PAYMENTS_NOTIFY, 'Necesitas seleccionar un método de pago')
        }
        if (getters.paidTotal <= 0){
            return dispatch(PAYMENTS_NOTIFY, 'Necesitas ingresar un monto a pagar')
        }
        return true
    },
    /**
     * [PAYMENTS_PROCESS]
     * @param {*} param0
     * @param {*} payload
     */
    [PAYMENTS_PROCESS] ({ getters, commit, dispatch }, payload ) {
        commit(SET_PROCESSING_PAYMENTS, true)
        const data = {
            jsonrpc: '2.0',
            method: 'call',
            params: {
                invoiceId: getters.selectedInvoice.id,
                journalId: getters.selectedJournal.id,
                customerId: getters.selectedCustomer.id,
                amountPayments: getters.paidTotal <= getters.paymentCurrencyTotal ? getters.paidTotal : getters.paymentCurrencyTotal,
                moveLines: getters.movesPayments.map(item => {
                    return item.id
                })
            }
        }

        return axios.post(PAYMENTS_PROCESS_URL, data).then(response => {
            commit(SET_PROCESSING_PAYMENTS, !response.data.result)
            dispatch(COMPLETED_PAYMENTS, response.data.result)
        }).catch(error => {
            console.log(error);
            commit(SET_PROCESSING_PAYMENTS, false)
        })
    },
    /**
     * [COMPLETED_PAYMENTS]
     *
     */
    [COMPLETED_PAYMENTS] ({ commit }, payload) {
        commit(SET_COMPLETED_PAYMENTS, !!payload)
    },
    /**
     *
     */
    [RESET_PAYMENT_CUSTOMER] ({ rootState, commit, dispatch }){
        for (let key in rootState) {
            if (!(rootState[key] instanceof Object)){
                continue
            }
            dispatch(`reset${key[0].toUpperCase()}${key.slice(1)}`)
        }
        dispatch(INIT_PAYMENTS)
    }
}

export default actions
