import { CHANGE_INITIAL_PAID, RESET_PAYMENTS, CALCULATE_CURRENCY_TOTAL } from '@/constants/actionTypes'
import { SET_PAID_TOTAL, SET_CALCULATE_CURRENCY_TOTAL, SET_CURRENCY_TOTAL } from '@/constants/mutationTypes'

const initialState = {
    paidTotal: 0,
    paymentsReturned: 0,
    paymentCurrencyTotal: 0
}
const state = {
    paidTotal: initialState.paidTotal,
    paymentsReturned: initialState.paymentsReturned,
    paymentCurrencyTotal: initialState.paymentCurrencyTotal
}
const getters = {
    /**
     * [paidTotal description]
     * @method paidTotal
     * @param  {[type]}     state [description]
     * @return {[type]}     [description]
     */
    paidTotal (state) {
        return state.paidTotal
    },
    /**
     * [paymentsReturned description]
     * @method paymentsReturned
     * @param  {[type]}         state [description]
     * @return {[type]}               [description]
     */
    paymentsReturned (state) {
        return state.paymentsReturned
    },
    /**
     * [paymentCurrencyTotal description]
     * @param  {[type]} state [description]
     * @return {[type]}       [description]
     */
    paymentCurrencyTotal (state) {
        return state.paymentCurrencyTotal
    }
}

const mutations = {
    /**
     * [paidTotal description]
     * @type {[type]}
     */
    [SET_PAID_TOTAL] (state, payload) {
        state.paidTotal = payload
    },
    /**
     * [paymentCurrencyTotal description]
     * @type {[type]}
     */
    [SET_CURRENCY_TOTAL] (state, payload) {
        state.paymentCurrencyTotal = payload
    },
    /**
     * [SET_CALCULATE_CURRENCY_TOTAL]
     * @type {[type]}
     */
    [SET_CALCULATE_CURRENCY_TOTAL] (state, payload) {
        let currencyJournal = payload.movesCurrencyAmount.find(item => item.id ===payload.currency.id)
        let total = currencyJournal.amount
        if (!currencyJournal)
            total = 0
        state.paymentCurrencyTotal = total
    }
}
const actions = {
    /**
     * [CHANGE_INITIAL_PAID]
     */
    [CHANGE_INITIAL_PAID] ({ commit },payload) {
        commit(SET_PAID_TOTAL, payload)
    },
    /**
     * [CALCULATE_CURRENCY_TOTAL]
     */
    [CALCULATE_CURRENCY_TOTAL] ({commit, getters}, payload) {
        let currency = []
        if (payload.currency.id){
            currency = payload.currency
        } else {
            currency = getters.currencyCompany
        }
        commit(SET_CALCULATE_CURRENCY_TOTAL, {
            currency: currency,
            movesCurrencyAmount: getters.currencyAmount
        })
        commit(SET_PAID_TOTAL, 0)
    },
    /**
     * [RESET_PAYMENTS]
     */
    [RESET_PAYMENTS] ({ commit }) {
        commit(SET_PAID_TOTAL, 0)
        commit(SET_CURRENCY_TOTAL, 0)
    }
}

export default {
    state,
    getters,
    mutations,
    actions
}
